@@ -126,14 +126,6 @@ CLUSTER_SORT_KEYS = [
     CLUSTER_INIT_AT, CLUSTER_CREATED_AT, CLUSTER_UPDATED_AT,
 ]
 
-DETECTION_TYPES = (
-    LIFECYCLE_EVENTS, NODE_STATUS_POLLING,
-    # LB_STATUS_POLLING,
-) = (
-    'LIFECYCLE_EVENTS', 'NODE_STATUS_POLLING',
-    # 'LB_STATUS_POLLING',
-)
-
 NODE_ATTRS = (
     NODE_INDEX, NODE_NAME, NODE_PROFILE_ID, NODE_CLUSTER_ID,
     NODE_INIT_AT, NODE_CREATED_AT, NODE_UPDATED_AT,
@@ -267,12 +259,19 @@ EVENT_LEVELS = {
     'DEBUG': logging.DEBUG,
 }
 
-RECOVER_OPERATIONS = (
-    RECOVER_RECREATE, RECOVER_REBUILD,
+DETECTION_TYPES = (
+    LIFECYCLE_EVENTS, NODE_STATUS_POLLING,
+    # LB_STATUS_POLLING,
 ) = (
-    'RECREATE', 'REBUILD',
+    'LIFECYCLE_EVENTS', 'NODE_STATUS_POLLING',
+    # 'LB_STATUS_POLLING',
 )
 
+RECOVERY_ACTIONS = (
+    RECOVER_REBOOT, RECOVER_REBUILD, RECOVER_RECREATE,
+) = (
+    'REBOOT', 'REBUILD', 'RECREATE',
+)
 
 NOTIFICATION_PRIORITIES = (
     PRIO_AUDIT, PRIO_CRITICAL, PRIO_ERROR, PRIO_WARN, PRIO_INFO, PRIO_DEBUG,
@@ -281,7 +280,6 @@ NOTIFICATION_PRIORITIES = (
     'audit', 'critical', 'error', 'warn', 'info', 'debug', 'sample',
 )
 
-
 NOTIFICATION_PHASES = (
     PHASE_START, PHASE_END, PHASE_ERROR,
 ) = (
