@@ -366,12 +366,10 @@ class Action(object):
         """
         reason = self.data['reason']
         if self.data['status'] == policy_mod.CHECK_OK:
-            EVENT.debug(self, 'check', reason)
             return True
 
-        reason = _("Failed policy '%(name)s': %(reason)s"
-                   ) % {'name': name, 'reason': reason}
-        EVENT.error(self, consts.PHASE_ERROR, reason)
+        self.data['reason'] = _("Failed policy '%(name)s': %(reason)s"
+                                ) % {'name': name, 'reason': reason}
         return False
 
     def policy_check(self, cluster_id, target):
