@@ -11,75 +11,31 @@
 # under the License.
 
 from oslo_context import context as oslo_context
+from oslo_utils import reflection
 from oslo_utils import timeutils
 
 from senlin.common import context as senlin_context
 from senlin.common import exception
 from senlin.common.i18n import _
 from senlin.common import schema
-from senlin.db import api as db_api
+from senlin.common import utils
+from senlin.drivers import base as driver
 from senlin.engine import environment
-
-# Default enforcement levels and level names.
-#
-#  - MUST: A policy of this enforcement level must be strictly checked. A
-#          violation of such a policy will lead a cluster to ``CRITICAL``
-#          status, which means that the cluster is in a problematic status
-#          that cannot be recovered using Senlin APIs. A manual intervention
-#          is needed. Such a cluster should not be treated as funtional any
-#          more.
-#
-#  - SHOULD: A violation of a policy at this enforcement level will render a
-#            cluster into an ``ERROR`` status. A manual intervention is needed
-#            to recover the cluster. The cluster may and may not be providing
-#            services in the ``ERROR`` status.
-#
-#  - WOULD: A policy of this enforcement level is usually about some
-#           operations that would be done when the policy is enforced. A
-#           violation of the policy will leave the cluster in a ``WARNING``
-#           status, which means that the cluster is still operational, but
-#           there are unsuccessful operations attempted.
-#
-#  - MIGHT: A policy of this enforcement level is usually associated with
-#           certain operations that may or may not be done. A violation of
-#           this policy will not cause any negative impact to the cluster.
-#
-
-POLICY_LEVELS = (
-    MUST, SHOULD, WOULD, MIGHT,
-) = (
-    50, 40, 30, 20
-)
-
-_levelNames = {
-    MUST: 'MUST',
-    SHOULD: 'SHOULD',
-    WOULD: 'WOULD',
-    MIGHT: 'MIGHT',
-    'MUST': MUST,
-    'SHOULD': SHOULD,
-    'WOULD': WOULD,
-    'MIGHT': MIGHT,
-}
+from senlin.objects import credential as co
+from senlin.objects import policy as po
 
 CHECK_RESULTS = (
-    CHECK_OK, CHECK_ERROR
+    CHECK_OK, CHECK_ERROR,
 ) = (
-    'OK', 'ERROR'
+    'OK', 'ERROR',
 )
 
 
-def getLevelName(level):
-    '''Get a level name or number.
-
-    Return a level name if given a numeric value; or return a value if given
-    a string.  If level is not predefined, "Level %s" will be returned.
-    '''
-    return _levelNames.get(level, ("Level %s" % level))
-
-
 class Policy(object):
-    '''Base class for policies.'''
+    """Base class for policies."""
+
+    VERSIONS = {}
+
     PROFILE_TYPE = 'ANY'
 
     KEYS = (
@@ -118,11 +74,12 @@ class Policy(object):
         :returns: An instance of a specific sub-class of Policy.
         """
         type_name, version = schema.get_spec_version(spec)
+        type_str = "-".join([type_name, version])
 
         if cls != Policy:
             PolicyClass = cls
         else:
-            PolicyClass = environment.global_env().get_policy(type_name)
+            PolicyClass = environment.global_env().get_policy(type_str)
 
         return super(Policy, cls).__new__(PolicyClass)
 
@@ -136,47 +93,52 @@ class Policy(object):
         """
 
         type_name, version = schema.get_spec_version(spec)
-
+        type_str = "-".join([type_name, version])
         self.name = name
         self.spec = spec
 
         self.id = kwargs.get('id', None)
-        self.type = kwargs.get('type', "%s-%s" % (type_name, version))
+        self.type = kwargs.get('type', type_str)
         self.user = kwargs.get('user')
         self.project = kwargs.get('project')
         self.domain = kwargs.get('domain')
-        self.level = kwargs.get('level', SHOULD)
-        self.cooldown = kwargs.get('cooldown', 0)
         self.data = kwargs.get('data', {})
 
-        self.created_time = kwargs.get('created_time', None)
-        self.updated_time = kwargs.get('updated_time', None)
-        self.deleted_time = kwargs.get('deleted_time', None)
+        self.created_at = kwargs.get('created_at', None)
+        self.updated_at = kwargs.get('updated_at', None)
 
         self.spec_data = schema.Spec(self.spec_schema, spec)
-        self.properties = schema.Spec(self.properties_schema,
-                                      self.spec.get(self.PROPERTIES, {}))
+        self.properties = schema.Spec(
+            self.properties_schema,
+            self.spec.get(self.PROPERTIES, {}),
+            version)
+
         self.singleton = True
+        self._novaclient = None
+        self._keystoneclient = None
+        self._networkclient = None
+        self._lbaasclient = None
 
     @classmethod
-    def _from_db_record(cls, record):
-        '''Construct a policy object from a database record.'''
+    def _from_object(cls, policy):
+        """Construct a policy from a Policy object.
+
+        @param cls: The target class.
+        @param policy: A policy object.
+        """
 
         kwargs = {
-            'id': record.id,
-            'type': record.type,
-            'user': record.user,
-            'project': record.project,
-            'domain': record.domain,
-            'level': record.level,
-            'cooldown': record.cooldown,
-            'created_time': record.created_time,
-            'updated_time': record.updated_time,
-            'deleted_time': record.deleted_time,
-            'data': record.data,
+            'id': policy.id,
+            'type': policy.type,
+            'user': policy.user,
+            'project': policy.project,
+            'domain': policy.domain,
+            'created_at': policy.created_at,
+            'updated_at': policy.updated_at,
+            'data': policy.data,
         }
 
-        return cls(record.name, record.spec, **kwargs)
+        return cls(policy.name, policy.spec, **kwargs)
 
     @classmethod
     def load(cls, context, policy_id=None, db_policy=None, project_safe=True):
@@ -191,36 +153,20 @@ class Policy(object):
         :returns: An object of the proper policy class.
         """
         if db_policy is None:
-            db_policy = db_api.policy_get(context, policy_id,
-                                          project_safe=project_safe)
+            db_policy = po.Policy.get(context, policy_id,
+                                      project_safe=project_safe)
             if db_policy is None:
-                raise exception.PolicyNotFound(policy=policy_id)
+                raise exception.ResourceNotFound(type='policy', id=policy_id)
 
-        return cls._from_db_record(db_policy)
-
-    @classmethod
-    def load_all(cls, context, limit=None, sort_keys=None, marker=None,
-                 sort_dir=None, filters=None, show_deleted=False,
-                 project_safe=True):
-        '''Retrieve all policies from database.'''
-
-        records = db_api.policy_get_all(context, limit=limit, marker=marker,
-                                        sort_keys=sort_keys,
-                                        sort_dir=sort_dir,
-                                        filters=filters,
-                                        show_deleted=show_deleted,
-                                        project_safe=project_safe)
-
-        for record in records:
-            yield cls._from_db_record(record)
+        return cls._from_object(db_policy)
 
     @classmethod
     def delete(cls, context, policy_id):
-        db_api.policy_delete(context, policy_id)
+        po.Policy.delete(context, policy_id)
 
     def store(self, context):
         '''Store the policy object into database table.'''
-        timestamp = timeutils.utcnow()
+        timestamp = timeutils.utcnow(True)
 
         values = {
             'name': self.name,
@@ -229,24 +175,22 @@ class Policy(object):
             'project': self.project,
             'domain': self.domain,
             'spec': self.spec,
-            'level': self.level,
-            'cooldown': self.cooldown,
             'data': self.data,
         }
 
         if self.id is not None:
-            self.updated_time = timestamp
-            values['updated_time'] = timestamp
-            db_api.policy_update(context, self.id, values)
+            self.updated_at = timestamp
+            values['updated_at'] = timestamp
+            po.Policy.update(context, self.id, values)
         else:
-            self.created_time = timestamp
-            values['created_time'] = timestamp
-            policy = db_api.policy_create(context, values)
+            self.created_at = timestamp
+            values['created_at'] = timestamp
+            policy = po.Policy.create(context, values)
             self.id = policy.id
 
         return self.id
 
-    def validate(self):
+    def validate(self, context, validate_props=False):
         '''Validate the schema and the data provided.'''
         self.spec_data.validate()
         self.properties.validate()
@@ -257,7 +201,7 @@ class Policy(object):
                     for name, schema in cls.properties_schema.items())
 
     def _build_policy_data(self, data):
-        clsname = self.__class__.__name__
+        clsname = reflection.get_class_name(self, fully_qualified=False)
         version = self.VERSION
         result = {
             clsname: {
@@ -268,7 +212,7 @@ class Policy(object):
         return result
 
     def _extract_policy_data(self, policy_data):
-        clsname = self.__class__.__name__
+        clsname = reflection.get_class_name(self, fully_qualified=False)
         if clsname not in policy_data:
             return None
         data = policy_data.get(clsname)
@@ -277,10 +221,88 @@ class Policy(object):
 
         return data.get('data', None)
 
-    def attach(self, cluster):
+    def _build_conn_params(self, user, project):
+        """Build trust-based connection parameters.
+
+        :param user: the user for which the trust will be checked.
+        :param object: the user for which the trust will be checked.
+        """
+        service_creds = senlin_context.get_service_credentials()
+        params = {
+            'username': service_creds.get('username'),
+            'password': service_creds.get('password'),
+            'auth_url': service_creds.get('auth_url'),
+            'user_domain_name': service_creds.get('user_domain_name')
+        }
+
+        cred = co.Credential.get(oslo_context.get_current(), user, project)
+        if cred is None:
+            raise exception.TrustNotFound(trustor=user)
+        params['trust_id'] = cred.cred['openstack']['trust']
+
+        return params
+
+    def keystone(self, user, project):
+        """Construct keystone client based on object.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the keystone client.
+        """
+        if self._keystoneclient is not None:
+            return self._keystoneclient
+        params = self._build_conn_params(user, project)
+        self._keystoneclient = driver.SenlinDriver().identity(params)
+        return self._keystoneclient
+
+    def nova(self, user, project):
+        """Construct nova client based on user and project.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the nova client.
+        """
+        if self._novaclient is not None:
+            return self._novaclient
+
+        params = self._build_conn_params(user, project)
+        self._novaclient = driver.SenlinDriver().compute(params)
+        return self._novaclient
+
+    def network(self, user, project):
+        """Construct network client based on user and project.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the network client.
+        """
+        if self._networkclient is not None:
+            return self._networkclient
+
+        params = self._build_conn_params(user, project)
+        self._networkclient = driver.SenlinDriver().network(params)
+        return self._networkclient
+
+    def lbaas(self, user, project):
+        """Construct LB service client based on user and project.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the LB service client.
+        """
+        if self._lbaasclient is not None:
+            return self._lbaasclient
+
+        params = self._build_conn_params(user, project)
+
+        self._lbaasclient = driver.SenlinDriver().loadbalancing(params)
+        return self._lbaasclient
+
+    def attach(self, cluster, enabled=True):
         '''Method to be invoked before policy is attached to a cluster.
 
-        :param cluster: the cluster to which the policy is being attached to.
+        :param cluster: The cluster to which the policy is being attached to.
+        :param enabled: The attached cluster policy is enabled or disabled.
         :returns: (True, message) if the operation is successful, or (False,
                  error) otherwise.
         '''
@@ -299,6 +321,15 @@ class Policy(object):
         '''Method to be invoked before policy is detached from a cluster.'''
         return True, None
 
+    def need_check(self, target, action):
+        if getattr(self, 'TARGET', None) is None:
+            return True
+
+        if (target, action.action) in self.TARGET:
+            return True
+        else:
+            return False
+
     def pre_op(self, cluster_id, action):
         '''A method that will be invoked before an action execution.'''
         return
@@ -308,9 +339,6 @@ class Policy(object):
         return
 
     def to_dict(self):
-        def _fmt_time(value):
-            return value and value.isoformat()
-
         pb_dict = {
             'id': self.id,
             'name': self.name,
@@ -319,32 +347,8 @@ class Policy(object):
             'project': self.project,
             'domain': self.domain,
             'spec': self.spec,
-            'level': self.level,
-            'cooldown': self.cooldown,
-            'created_time': _fmt_time(self.created_time),
-            'updated_time': _fmt_time(self.updated_time),
-            'deleted_time': _fmt_time(self.deleted_time),
+            'created_at': utils.isotime(self.created_at),
+            'updated_at': utils.isotime(self.updated_at),
             'data': self.data,
         }
         return pb_dict
-
-    def _build_conn_params(self, cluster):
-        """Build trust-based connection parameters.
-
-        :param cluster: the cluste for which the trust will be checked.
-        """
-        service_creds = senlin_context.get_service_context()
-        params = {
-            'username': service_creds.get('username'),
-            'password': service_creds.get('password'),
-            'auth_url': service_creds.get('auth_url'),
-            'user_domain_name': service_creds.get('user_domain_name')
-        }
-
-        cred = db_api.cred_get(oslo_context.get_current(),
-                               cluster.user, cluster.project)
-        if cred is None:
-            raise exception.TrustNotFound(trustor=cluster.user)
-        params['trust_id'] = [cred.cred['openstack']['trust']]
-
-        return params
